/*
 *  display a html file by calling netscape with -remote option
 *
 *  RCS:
 *      $Revision: 1.1.1.1 $
 *      $Date: 2001/08/12 05:39:20 $
 *
 *  Description:
 *      This program calls netscape with -remote 'openURL(URL)'.
 *      The program finds out the absolute path of the file. For example,
 *      file can be specified as ./file, ../file /a/b/c file. netscape
 *      wants absolute path.
 *
 *  Limitations and Comments:
 *      The fileutils routines are adapted from nedit source code.
 *
 *
 *  Development History:
 *      who                  when       why
 *      ma_muquit@fccc.edu   May-30-97  first cut
 *                           May-30-97  acting weired on sprintf, used
 *                                      save. works fine now on linux.
 *      muquit@muquit.com    Aug-11-2001 netscape does not work anymore with
 *         KDE2 with -remote. I've not idea whose bug is this. It seems if 
 *         id of the window is passed it works fine. So I wrote a routine 
 *         to find the "Navigator" id. Now if window id is 0, it is first
 *         time.
 *         
 */

#include "vhtml.h"


/* private protos */
static void usage(char *);
static int execProg(char *progname,char **arglist);


int main(int argc,char **argv)
{
    char
        name[MAXPATHLEN],
        namepath[MAXPATHLEN];

    char
        idbuf[100],
        buf[MAXPATHLEN];
        
    char
        **s,
        **namelist;

    int
        code,
        rc;

    int
        ns_up=0;

    char
        *save;

    unsigned int
        window;

    if (argc != 2)
    {
        usage(argv[0]);
        /* won't be here */
    }



    
    rc=parseFilename(argv[1],name,namepath);
    if (rc != 0)
    {
        (void) fprintf (stderr,"Unable to parse filename: %s\n",argv[1]);
        exit(1);
    }

#ifdef DEBUG
    (void) fprintf (stderr,"name=%s\n",name);
    (void) fprintf (stderr,"namepath=%s\n",namepath);
#endif

    if (strncasecmp(argv[1],"http://",7) == 0)
    {
        (void) strcpy(buf,argv[1]);
        save=strdup(buf);
        if (save == (char *) NULL)
        {
            (void) fprintf (stderr,"malloc failed!\n");
            exit(1);
        }
    }
    else
    {
        (void) sprintf(buf,"file:%s%s",namepath,name);
        save=strdup(buf);
        if (save == (char *) NULL)
        {
            (void) fprintf (stderr,"malloc failed!\n");
            exit(1);
        }
    }
    (void) sprintf(buf,"openURL(%s)",save);

#ifdef DEBUG
    (void) fprintf (stderr,"buf=%s\n",buf);
#endif

    /* build the array for execv */
    namelist=(char **) malloc(strlen(buf)*sizeof(char *) + 100);
    if (namelist == (char **) NULL)
    {
        (void) fprintf (stderr,"memory allocation problem\n");
        exit(1);
    }

    /* find the Navigator window id from class hint name */
    window=findWindowId("Navigator");
    /*
    (void) fprintf(stderr,"Navigator id: 0x%lx\n",rc);
    */

    if (window == 0) /* first time */
    {
        s=namelist;
        *s++ = "netscape";
        *s++ = save;
        *s='\0';
    }
    else
    {
        s=namelist;
        *s++ = "netscape";
        *s++ = "-id";
        (void) sprintf(idbuf,"0x%lx",window);
        *s++ = idbuf;
        *s++ = "-remote";
        *s++ = buf;
        *s='\0';
    }

    if (window == 0)
    {
        (void) fprintf(stderr," Starting a new copy of netscape..please wait\n");
        if (fork() != 0)
            exit(0);
        execv(NETSCAPE_PATH,namelist);
    }
    else
    {
        rc=execProg(NETSCAPE_PATH,namelist);
    }
    goto ExitProcessing;


    rc=execProg(NETSCAPE_PATH,namelist);
    if (LOW_BYTE(rc) == 0)
    {
        if ((code=HIGH_BYTE(rc)) != 0)
        {
            s=namelist;
            *s++ = "netscape";
            *s++ = save;
            *s='\0';
            /* netscape is not up, execute directly */
            (void) fprintf (stderr,
                "Starting a fresh copy of netscape..Please Wait!\n");
            /* 
            ** let the parent go, otherwise we'r not gonna get the prompt
            ** back
            */
            if (fork() != 0)
                exit(0);
            execv(NETSCAPE_PATH,namelist);
            (void) fprintf (stderr,"We should not be here!\n");
        }
    }
    else
    {
        (void) fprintf (stderr,"seg fault may be!\n");
    }
#ifdef DEBUG
    (void) fprintf (stderr,"satus code from execProg()=%d\n",rc);
#endif
   
ExitProcessing:

    (void) free ((char *) save);
    exit(0);
}

/* usage */
static void usage(char *progname)
{

    (void) fprintf (stderr,"viewhtml %s by Muhammad A Muquit\n",
        VIEW_HTML_VERSION);
    (void) fprintf (stderr,"A program to call netscape with -remote\n\n");
    (void) fprintf (stderr,"Usage: %s <filename>\n",progname);
    exit(1);
}



/*
 *  execProg -- executes a program and returns the exit code of the program
 *
 *  Parameters:
 *      char    *progpath   path of the program to execute
 *      char    **arglist   tokenized arg list of the program
 *  Return Values:
 *      exit code of the executed program       on success
 *      -1      on failure
 *  Limitations and Comments:
 *      none
 *
 *
 *  Development History:
 *      who                  when       why
 *      ma_muquit@fccc.edu   May-30-97  first cut
 */

static int execProg(char *progpath,char **arglist)
{
    int
        status,
        code;

    switch(fork())
    {
        case -1:
        {
            (void) fprintf (stderr,"Unable to create new process\n");
            return (-1);
            break;  /* won't be here */
        }

        case 0:
        {
            execvp(progpath,arglist);
            (void) fprintf (stderr,"Exec failed!\n");
            exit(1);
            break;  /* won't be here */
        }

        default:
        {
            if (wait(&status) == -1)
            {
                perror("wait");
                exit(-1);
            }
            return (status);
        }
    }

    return (0);
}
